#!/bin/bash

# Solr Java Heap Space for Search/Registry Service
# Only update if heap size is too large for machine it is running on
SOLR_HEAP="1024M"

#################################################################################
# WARNING: You should not need to update below. Modify at your own risk.        #
#################################################################################

usage() {
    echo "Usage: $0 install/upgrade" </path/to/harvest/output>" 1>&2
    echo "       </path/to/harvest/output> - Should match harvest execution output path."
    exit 1
}

check_exec() {
    # Check if it ran successfully
    if [ $1 -eq 0 ]; then
	echo "SUCCESS"
    else
	echo "FAILED"
	echo "See $LOG"
	echo
	exit 1
    fi
}

check_status() {
    status=$(echo $1 | tr -d '\n' | awk -F"status" '{print $2}' | awk -F, '{print $1}' | awk -F: '{print $2}')
    check_exec $((status))
}

if [[ $# -lt 1 ]]; then
    usage
fi

# Check if the JAVA_HOME environment variable is set.
if [ -z "${JAVA_HOME}" ]; then
   echo "The JAVA_HOME environment variable is not set." 1>&2
   exit 1
fi

# Solr Default configs for SolrCloud
maxShardsPerNode=2
numShards=1
replicationFactor=1

# Get path to this base dir where script exists
DIR=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )
LOG=${DIR}/search-deploy-log-$(date '+%Y%m%d_%H%M%S').txt
DOCK_IMAGE=search-service

if [ $1 == "install" ]; then
    install=true
elif [ $1 == "upgrade" ]; then
    install=false
else
    usage
fi

if [ -n "$2" ]; then
    basepath=$2
else
    basepath=$(pwd)
fi

# TODO: Add some output logs and ask some questions before proceeding

version=$(cat ${DIR}/../VERSION.txt)
#echo "Version: $version"

# Build the new Docker image
echo -ne "Building Registry/Search Docker Image.      " | tee -a $LOG
cd $DIR/../build
docker build -t $DOCK_IMAGE:$version -f Dockerfile-win ../ >>$LOG 2>&1
cd ../../

check_exec $?

# TODO: Stop and rename old container for upgrade

docker volume create registry-search-service_zoo_data > /dev/null
docker volume create registry-search-service_pds_shard1_replica_n1 > /dev/null
docker volume create registry-search-service_system_shard1_replica_n1 > /dev/null


# Start up container. TODO: clean this up, maybe use docker-compose
echo -ne "Starting Registry/Search Docker Container.  " | tee -a $LOG
docker run --name $DOCK_IMAGE -u solr \
-v $basepath/solr-docs:/data/solr-docs \
-v registry-search-service_zoo_data:/opt/solr/server/solr/zoo_data/ \
-v registry-search-service_pds_shard1_replica_n1:/opt/solr/server/solr/pds_shard1_replica_n1 \
-v registry-search-service_system_shard1_replica_n1:/opt/solr/server/solr/.system_shard1_replica_n1 \
-d -p 8983:8983 -e SOLR_HEAP=$SOLR_HEAP $DOCK_IMAGE:$version \
/bin/bash -c 'chown solr /opt/solr/server/solr/zoo_data/ /opt/solr/server/solr/pds_shard1_replica_n1 /opt/solr/server/solr/.system_shard1_replica_n1; solr start -c -Djavax.xml.transform.TransformerFactory=net.sf.saxon.TransformerFactoryImpl -Dsolr.pds.home=/opt/solr/server/solr/ -f' >>$LOG 2>&1
    
check_exec $?

# TODO: Add some checks to make sure the above executed successfully

if $install; then
    # Wait 30 seconds for Solr server to start
    echo "Waiting for Solr server to start..." | tee -a $LOG
    sleep 30

    # Create the Registry collection
    echo -ne "Creating the Registry collection.         " | tee -a $LOG
    check=$(curl "http://localhost:8983/solr/admin/collections?action=CREATE&name=.system&maxShardsPerNode=${maxShardsPerNode}&numShards=${numShards}&replicationFactor=${replicationFactor}" 2>>$LOG | tee -a $LOG)
    curl "http://localhost:8983/solr/admin/collections?action=CREATEALIAS&name=registry&collections=.system" >>$LOG 2>&1

    check_status "$check"

    # Create the Search collection
    echo -ne "Creating the Search collection.           " | tee -a $LOG
    docker exec -it --user=solr $DOCK_IMAGE solr create -c pds -d pds -s ${numShards} -rf ${replicationFactor} >>$LOG 2>&1
    check_exec $?
else
    # TODO: How to handle upgrades
    echo "TODO"
    # TODO: Stop currently running solr instance if one is running
    # TODO: Check if this version already exists
    # TODO: If not, start this new version with "hopefully" the same data
    #       Check the data paths exist already.
fi

docker exec -it $DOCK_IMAGE solr status >>$LOG 2>&1

exit 0
